@@ -70,27 +70,30 @@ with `"x-"` (vendor extension).
 
 """
 
+# pyright: reportIncompatibleMethodOverride=false
 from __future__ import annotations
 
-import warnings
 import typing
+import warnings
 
 from marshmallow import Schema
+from packaging.version import Version
 
-from apispec import BasePlugin, APISpec
-from apispec.utils import OpenAPIVersion
-from .common import resolve_schema_instance, make_schema_key, resolve_schema_cls
+from apispec import APISpec, BasePlugin
+
+from .common import make_schema_key, resolve_schema_cls, resolve_schema_instance
 from .openapi import OpenAPIConverter
 from .schema_resolver import SchemaResolver
 
 
 def resolver(schema: type[Schema]) -> str:
     """Default schema name resolver function that strips 'Schema' from the end of the class name."""
-    schema_cls = resolve_schema_cls(schema)
+    resolved = resolve_schema_cls(schema)
+    schema_cls = resolved[0] if isinstance(resolved, list) else resolved
     name = schema_cls.__name__
     if name.endswith("Schema"):
-        return name[:-6] or name
-    return name
+        name = name[:-6] or name
+    return name.strip()
 
 
 class MarshmallowPlugin(BasePlugin):
@@ -120,7 +123,7 @@ class MarshmallowPlugin(BasePlugin):
         super().__init__()
         self.schema_name_resolver = schema_name_resolver or resolver
         self.spec: APISpec | None = None
-        self.openapi_version: OpenAPIVersion | None = None
+        self.openapi_version: Version | None = None
         self.converter: OpenAPIConverter | None = None
         self.resolver: SchemaResolver | None = None
 
@@ -137,8 +140,10 @@ class MarshmallowPlugin(BasePlugin):
             openapi_version=spec.openapi_version, converter=self.converter
         )
 
-    def map_to_openapi_type(self, *args):
-        """Decorator to set mapping for custom fields.
+    def map_to_openapi_type(self, field_cls, *args):
+        """Set mapping for custom field class.
+
+        :param type field_cls: Field class to set mapping for.
 
         ``*args`` can be:
 
@@ -147,15 +152,20 @@ class MarshmallowPlugin(BasePlugin):
 
         Examples: ::
 
-            @ma_plugin.map_to_openapi_type('string', 'uuid')
-            class MyCustomField(Integer):
+            # Override Integer mapping
+            class Int32(Integer):
                 # ...
 
-            @ma_plugin.map_to_openapi_type(Integer)  # will map to ('integer', None)
-            class MyCustomFieldThatsKindaLikeAnInteger(Integer):
+            ma_plugin.map_to_openapi_type(Int32, 'string', 'int32')
+
+            # Map to ('integer', None) like Integer
+            class IntegerLike(Integer):
                 # ...
+
+            ma_plugin.map_to_openapi_type(IntegerLike, Integer)
         """
-        return self.converter.map_to_openapi_type(*args)
+        assert self.converter is not None, "init_spec has not yet been called"
+        return self.converter.map_to_openapi_type(field_cls, *args)
 
     def schema_helper(self, name, _, schema=None, **kwargs):
         """Definition helper that allows using a marshmallow
@@ -171,6 +181,7 @@ class MarshmallowPlugin(BasePlugin):
 
         schema_key = make_schema_key(schema_instance)
         self.warn_if_schema_already_in_spec(schema_key)
+        assert self.converter is not None, "init_spec has not yet been called"
         self.converter.refs[schema_key] = name
 
         json_schema = self.converter.schema2jsonschema(schema_instance)
@@ -184,6 +195,7 @@ class MarshmallowPlugin(BasePlugin):
         :param dict parameter: parameter fields. May contain a marshmallow
             Schema class or instance.
         """
+        assert self.resolver is not None, "init_spec has not yet been called"
         self.resolver.resolve_schema(parameter)
         return parameter
 
@@ -194,6 +206,7 @@ class MarshmallowPlugin(BasePlugin):
         :param dict parameter: response fields. May contain a marshmallow
             Schema class or instance.
         """
+        assert self.resolver is not None, "init_spec has not yet been called"
         self.resolver.resolve_response(response)
         return response
 
@@ -217,14 +230,15 @@ class MarshmallowPlugin(BasePlugin):
         assert self.resolver  # needed for mypy
         self.resolver.resolve_operations(operations)
 
-    def warn_if_schema_already_in_spec(self, schema_key: str) -> None:
+    def warn_if_schema_already_in_spec(self, schema_key: tuple) -> None:
         """Method to warn the user if the schema has already been added to the
         spec.
         """
         assert self.converter  # needed for mypy
         if schema_key in self.converter.refs:
             warnings.warn(
-                "{} has already been added to the spec. Adding it twice may "
-                "cause references to not resolve properly.".format(schema_key[0]),
+                f"{schema_key[0]} has already been added to the spec. Adding it twice may "
+                "cause references to not resolve properly.",
                 UserWarning,
+                stacklevel=2,
             )
