@@ -6,19 +6,21 @@
     This module is treated as private API.
     Users should not need to use this module directly.
 """
-import re
+
+from __future__ import annotations
+
 import functools
 import operator
+import re
 import typing
 import warnings
-from packaging.version import Version
 
 import marshmallow
 from marshmallow.orderedset import OrderedSet
-
+from packaging.version import Version
 
 # marshmallow field => (JSON Schema type, format)
-DEFAULT_FIELD_MAPPING = {
+DEFAULT_FIELD_MAPPING: dict[type, tuple[str | None, str | None]] = {
     marshmallow.fields.Integer: ("integer", None),
     marshmallow.fields.Number: ("number", None),
     marshmallow.fields.Float: ("number", None),
@@ -36,6 +38,9 @@ DEFAULT_FIELD_MAPPING = {
     marshmallow.fields.Field: (None, None),
     marshmallow.fields.Raw: (None, None),
     marshmallow.fields.List: ("array", None),
+    marshmallow.fields.IP: ("string", "ip"),
+    marshmallow.fields.IPv4: ("string", "ipv4"),
+    marshmallow.fields.IPv6: ("string", "ipv6"),
 }
 
 
@@ -86,7 +91,7 @@ _VALID_PREFIX = "x-"
 class FieldConverterMixin:
     """Adds methods for converting marshmallow fields to an OpenAPI properties."""
 
-    field_mapping = DEFAULT_FIELD_MAPPING
+    field_mapping: dict[type, tuple[str | None, str | None]] = DEFAULT_FIELD_MAPPING
     openapi_version: Version
 
     def init_attribute_functions(self):
@@ -98,7 +103,6 @@ class FieldConverterMixin:
             self.field2choices,
             self.field2read_only,
             self.field2write_only,
-            self.field2nullable,
             self.field2range,
             self.field2length,
             self.field2pattern,
@@ -109,6 +113,8 @@ class FieldConverterMixin:
             self.list2properties,
             self.dict2properties,
             self.timedelta2properties,
+            self.datetime2properties,
+            self.field2nullable,
         ]
 
     def map_to_openapi_type(self, field_cls, *args):
@@ -189,10 +195,9 @@ class FieldConverterMixin:
                 break
         else:
             warnings.warn(
-                "Field of type {} does not inherit from marshmallow.Field.".format(
-                    type(field)
-                ),
+                f"Field of type {type(field)} does not inherit from marshmallow.Field.",
                 UserWarning,
+                stacklevel=2,
             )
             type_, fmt = "string", None
 
@@ -252,6 +257,11 @@ class FieldConverterMixin:
             if choices:
                 attributes["enum"] = list(functools.reduce(operator.and_, choices))
 
+        if field.allow_none:
+            enum = attributes.get("enum")
+            if enum is not None and None not in enum:
+                attributes["enum"].append(None)
+
         return attributes
 
     def field2read_only(
@@ -292,8 +302,15 @@ class FieldConverterMixin:
                 attributes["x-nullable"] = True
             elif self.openapi_version.minor < 1:
                 attributes["nullable"] = True
+                if "$ref" in ret:
+                    attributes["allOf"] = [{"$ref": ret.pop("$ref")}]
             else:
-                attributes["type"] = [*make_type_list(ret.get("type")), "null"]
+                if "$ref" in ret:
+                    attributes["anyOf"] = [{"$ref": ret.pop("$ref")}, {"type": "null"}]
+                elif "allOf" in ret:
+                    attributes["anyOf"] = [*ret.pop("allOf"), {"type": "null"}]
+                else:
+                    attributes["type"] = [*make_type_list(ret.get("type")), "null"]
         return attributes
 
     def field2range(self, field: marshmallow.fields.Field, ret) -> dict:
@@ -380,9 +397,10 @@ class FieldConverterMixin:
 
         if next(regex_validators, None) is not None:
             warnings.warn(
-                "More than one regex validator defined on {} field. Only the "
-                "first one will be used in the output spec.".format(type(field)),
+                f"More than one regex validator defined on {type(field)} field. Only the "
+                "first one will be used in the output spec.",
                 UserWarning,
+                stacklevel=2,
             )
 
         return attributes
@@ -515,6 +533,55 @@ class FieldConverterMixin:
             else:
                 choices = (m.value for m in field.enum)
             ret["enum"] = [field.field._serialize(v, None, None) for v in choices]
+            if field.allow_none and None not in ret["enum"]:
+                ret["enum"].append(None)
+        return ret
+
+    def datetime2properties(self, field, **kwargs: typing.Any) -> dict:
+        """Return a dictionary of properties from :class:`DateTime <marshmallow.fields.DateTime` fields.
+
+        :param Field field: A marshmallow field.
+        :rtype: dict
+        """
+        ret = {}
+        if isinstance(field, marshmallow.fields.DateTime) and not isinstance(
+            field, marshmallow.fields.Date
+        ):
+            if field.format == "iso" or field.format is None:
+                # Will return { "type": "string", "format": "date-time" }
+                # as specified inside DEFAULT_FIELD_MAPPING
+                pass
+            elif field.format == "rfc":
+                ret = {
+                    "type": "string",
+                    "format": None,
+                    "example": "Wed, 02 Oct 2002 13:00:00 GMT",
+                    "pattern": r"((Mon|Tue|Wed|Thu|Fri|Sat|Sun), ){0,1}\d{2} "
+                    + r"(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d{4} \d{2}:\d{2}:\d{2} "
+                    + r"(UT|GMT|EST|EDT|CST|CDT|MST|MDT|PST|PDT|(Z|A|M|N)|(\+|-)\d{4})",
+                }
+            elif field.format == "timestamp":
+                ret = {
+                    "type": "number",
+                    "format": "float",
+                    "example": "1676451245.596",
+                    "min": "0",
+                }
+            elif field.format == "timestamp_ms":
+                ret = {
+                    "type": "number",
+                    "format": "float",
+                    "example": "1676451277514.654",
+                    "min": "0",
+                }
+            else:
+                ret = {
+                    "type": "string",
+                    "format": None,
+                    "pattern": field.metadata["pattern"]
+                    if field.metadata.get("pattern")
+                    else None,
+                }
         return ret
 
 
