@@ -3,9 +3,9 @@ import re
 from enum import Enum
 
 import pytest
-from marshmallow import fields, validate
+from marshmallow import Schema, fields, validate
 
-from .schemas import CategorySchema, CustomList, CustomStringField, CustomIntegerField
+from .schemas import CategorySchema, CustomIntegerField, CustomList, CustomStringField
 from .utils import build_ref, get_schemas
 
 
@@ -32,6 +32,9 @@ def test_field2choices_preserving_order(openapi):
         (fields.TimeDelta, "integer"),
         (fields.Email, "string"),
         (fields.URL, "string"),
+        (fields.IP, "string"),
+        (fields.IPv4, "string"),
+        (fields.IPv6, "string"),
         # Custom fields inherit types from their parents
         (CustomStringField, "string"),
         (CustomIntegerField, "integer"),
@@ -66,6 +69,9 @@ def test_formatted_field_translates_to_array(ListClass, spec_fixture):
         (fields.Date, "date"),
         (fields.Email, "email"),
         (fields.URL, "url"),
+        (fields.IP, "ip"),
+        (fields.IPv4, "ipv4"),
+        (fields.IPv6, "ipv6"),
     ],
 )
 def test_field2property_formats(FieldClass, expected_format, spec_fixture):
@@ -122,6 +128,22 @@ def test_field_with_choices(spec_fixture):
     assert set(res["enum"]) == {"freddie", "brian", "john"}
 
 
+def test_field_with_nullable_choices(spec_fixture):
+    field = fields.Str(
+        validate=validate.OneOf(["freddie", "brian", "john"]), allow_none=True
+    )
+    res = spec_fixture.openapi.field2property(field)
+    assert set(res["enum"]) == {"freddie", "brian", "john", None}
+
+
+def test_field_with_nullable_choices_returns_only_one_none(spec_fixture):
+    field = fields.Str(
+        validate=validate.OneOf(["freddie", "brian", "john", None]), allow_none=True
+    )
+    res = spec_fixture.openapi.field2property(field)
+    assert res["enum"] == ["freddie", "brian", "john", None]
+
+
 def test_field_with_equal(spec_fixture):
     field = fields.Str(validate=validate.Equal("only choice"))
     res = spec_fixture.openapi.field2property(field)
@@ -177,6 +199,43 @@ def test_field_with_allow_none(spec_fixture):
         assert res["type"] == ["string", "null"]
 
 
+@pytest.mark.parametrize("spec_fixture", ("2.0", "3.0.0", "3.1.0"), indirect=True)
+def test_nested_nullable(spec_fixture):
+    class Child(Schema):
+        name = fields.Str()
+
+    field = fields.Nested(Child, allow_none=True)
+    res = spec_fixture.openapi.field2property(field)
+    version = spec_fixture.openapi.openapi_version
+    if version.major < 3:
+        assert res == {"$ref": "#/definitions/Child", "x-nullable": True}
+    elif version.major == 3 and version.minor < 1:
+        assert res == {
+            "nullable": True,
+            "allOf": [{"$ref": "#/components/schemas/Child"}],
+        }
+    else:
+        assert res == {
+            "anyOf": [{"$ref": "#/components/schemas/Child"}, {"type": "null"}]
+        }
+
+
+@pytest.mark.parametrize("spec_fixture", ("2.0", "3.0.0", "3.1.0"), indirect=True)
+def test_nullable_pluck(spec_fixture):
+    class Example(Schema):
+        name = fields.Str()
+
+    field = fields.Pluck(Example, "name", allow_none=True)
+    res = spec_fixture.openapi.field2property(field)
+    version = spec_fixture.openapi.openapi_version
+    if version.major < 3:
+        assert res == {"type": "string", "x-nullable": True}
+    elif version.major == 3 and version.minor < 1:
+        assert res == {"type": "string", "nullable": True}
+    else:
+        assert res == {"type": ["string", "null"]}
+
+
 def test_field_with_dump_only(spec_fixture):
     field = fields.Str(dump_only=True)
     res = spec_fixture.openapi.field2property(field)
@@ -305,6 +364,27 @@ def test_enum_value_field(spec_fixture, by_value):
     assert ret["enum"] == [1, 2]
 
 
+def test_nullable_enum(spec_fixture):
+    class MyEnum(Enum):
+        one = 1
+        two = 2
+
+    field = fields.Enum(MyEnum, allow_none=True, by_value=True)
+    ret = spec_fixture.openapi.field2property(field)
+    assert ret["enum"] == [1, 2, None]
+
+
+def test_nullable_enum_returns_only_one_none(spec_fixture):
+    class MyEnum(Enum):
+        one = 1
+        two = 2
+        three = None
+
+    field = fields.Enum(MyEnum, allow_none=True, by_value=True)
+    ret = spec_fixture.openapi.field2property(field)
+    assert ret["enum"] == [1, 2, None]
+
+
 def test_field2property_nested_spec_metadatas(spec_fixture):
     spec_fixture.spec.components.schema("Category", schema=CategorySchema)
     category = fields.Nested(
@@ -377,6 +457,75 @@ def test_nested_field_with_property(spec_fixture):
     }
 
 
+def test_datetime2property_iso(spec_fixture):
+    field = fields.DateTime(format="iso")
+    res = spec_fixture.openapi.field2property(field)
+    assert res == {
+        "type": "string",
+        "format": "date-time",
+    }
+
+
+def test_datetime2property_rfc(spec_fixture):
+    field = fields.DateTime(format="rfc")
+    res = spec_fixture.openapi.field2property(field)
+    assert res == {
+        "type": "string",
+        "format": None,
+        "example": "Wed, 02 Oct 2002 13:00:00 GMT",
+        "pattern": r"((Mon|Tue|Wed|Thu|Fri|Sat|Sun), ){0,1}\d{2} "
+        + r"(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec) \d{4} \d{2}:\d{2}:\d{2} "
+        + r"(UT|GMT|EST|EDT|CST|CDT|MST|MDT|PST|PDT|(Z|A|M|N)|(\+|-)\d{4})",
+    }
+
+
+def test_datetime2property_timestamp(spec_fixture):
+    field = fields.DateTime(format="timestamp")
+    res = spec_fixture.openapi.field2property(field)
+    assert res == {
+        "type": "number",
+        "format": "float",
+        "min": "0",
+        "example": "1676451245.596",
+    }
+
+
+def test_datetime2property_timestamp_ms(spec_fixture):
+    field = fields.DateTime(format="timestamp_ms")
+    res = spec_fixture.openapi.field2property(field)
+    assert res == {
+        "type": "number",
+        "format": "float",
+        "min": "0",
+        "example": "1676451277514.654",
+    }
+
+
+def test_datetime2property_custom_format(spec_fixture):
+    field = fields.DateTime(
+        format="%d-%m%Y %H:%M:%S",
+        metadata={
+            "pattern": r"^((?:(\d{4}-\d{2}-\d{2})T(\d{2}:\d{2}:\d{2}(?:\.\d+)?))(Z|[\+-]\d{2}:\d{2})?)$"
+        },
+    )
+    res = spec_fixture.openapi.field2property(field)
+    assert res == {
+        "type": "string",
+        "format": None,
+        "pattern": r"^((?:(\d{4}-\d{2}-\d{2})T(\d{2}:\d{2}:\d{2}(?:\.\d+)?))(Z|[\+-]\d{2}:\d{2})?)$",
+    }
+
+
+def test_datetime2property_custom_format_missing_regex(spec_fixture):
+    field = fields.DateTime(format="%d-%m%Y %H:%M:%S")
+    res = spec_fixture.openapi.field2property(field)
+    assert res == {
+        "type": "string",
+        "format": None,
+        "pattern": None,
+    }
+
+
 class TestField2PropertyPluck:
     @pytest.fixture(autouse=True)
     def _setup(self, spec_fixture):
